/* Intrinsics for Loongson csr operations.

   Copyright (C) 2019 Free Software Foundation, Inc.
   Contributed by xuchenghua@loongson.cn.

   This file is part of GCC.

   GCC is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published
   by the Free Software Foundation; either version 3, or (at your
   option) any later version.

   GCC is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
   or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
   License for more details.

   Under Section 7 of GPL version 3, you are granted additional
   permissions described in the GCC Runtime Library Exception, version
   3.1, as published by the Free Software Foundation.

   You should have received a copy of the GNU General Public License and
   a copy of the GCC Runtime Library Exception along with this program;
   see the files COPYING3 and COPYING.RUNTIME respectively.  If not, see
   <http://www.gnu.org/licenses/>.  */

#ifndef _GCC_LOONGSON_CSR_H
#define _GCC_LOONGSON_CSR_H

#ifdef __cplusplus
extern "C" {
#endif

typedef struct drdtime{
	unsigned long dvalue;
	unsigned long dtimeid;
} __drdtime_t;

typedef struct rdtime{
	unsigned int value;
	unsigned int timeid;
} __rdtime_t;


#define __lcsr_cpucfg	__builtin_loongson_cpucfg
#define __lcsr_csrrd	__builtin_loongson_csrrd
#define __lcsr_csrwr	__builtin_loongson_csrwr
#define __lcsr_gcsrrd	__builtin_loongson_gcsrrd
#define __lcsr_gcsrwr	__builtin_loongson_gcsrwr
#define __lcsr_csrxchg __builtin_loongson_csrxchg
#define __lcsr_gcsrxchg __builtin_loongson_gcsrxchg
#define __lcsr_dcsrrd	__builtin_loongson_dcsrrd
#define __lcsr_dcsrwr	__builtin_loongson_dcsrwr
#define __lcsr_dgcsrrd	__builtin_loongson_dgcsrrd
#define __lcsr_dgcsrwr	__builtin_loongson_dgcsrwr
#define __lcsr_dcsrxchg __builtin_loongson_dcsrxchg
#define __lcsr_dgcsrxchg __builtin_loongson_dgcsrxchg
#define __lcsr_iocsrrd_b	__builtin_loongson_iocsrrd_b
#define __lcsr_iocsrrd_h	__builtin_loongson_iocsrrd_h
#define __lcsr_iocsrrd_w	__builtin_loongson_iocsrrd_w
#define __lcsr_iocsrrd_d	__builtin_loongson_iocsrrd_d
#define __lcsr_iocsrwr_b	__builtin_loongson_iocsrwr_b
#define __lcsr_iocsrwr_h	__builtin_loongson_iocsrwr_h
#define __lcsr_iocsrwr_w	__builtin_loongson_iocsrwr_w
#define __lcsr_iocsrwr_d	__builtin_loongson_iocsrwr_d
#define __lcsr_cacop	__builtin_loongson_cacop
#define __lcsr_dcacop	__builtin_loongson_dcacop
#define __lcsr_rdtime_d	__builtin_loongson_rdtime_d
#define __lcsr_rdtimel_w	__builtin_loongson_rdtimel_w
#define __lcsr_rdtimeh_w	__builtin_loongson_rdtimeh_w
#define __lcsr_lddir	__builtin_loongson_lddir
#define __lcsr_lwdir	__builtin_loongson_lwdir
#define __lcsr_ldpte	__builtin_loongson_ldpte
#define __lcsr_lwpte	__builtin_loongson_lwpte


extern __inline __drdtime_t __attribute__((__gnu_inline__, __always_inline__, __artificial__))
__builtin_loongson_rdtime_d (void)
{
  __drdtime_t drdtime;
  __asm__ volatile (
    "rdtime.d\t%[val],%[tid]\n\t"
    : [val]"=&r"(drdtime.dvalue),[tid]"=&r"(drdtime.dtimeid)
    :
  );
  return drdtime;
}

extern __inline __rdtime_t __attribute__((__gnu_inline__, __always_inline__, __artificial__))
__builtin_loongson_rdtimeh_w (void)
{
  __rdtime_t rdtime;
  __asm__ volatile (
    "rdtimeh.w\t%[val],%[tid]\n\t"
    : [val]"=&r"(rdtime.value),[tid]"=&r"(rdtime.timeid)
    :
  );
  return rdtime;
}

extern __inline __rdtime_t __attribute__((__gnu_inline__, __always_inline__, __artificial__))
__builtin_loongson_rdtimel_w (void)
{
  __rdtime_t rdtime;
  __asm__ volatile (
    "rdtimel.w\t%[val],%[tid]\n\t"
    : [val]"=&r"(rdtime.value),[tid]"=&r"(rdtime.timeid)
    :
  );
  return rdtime;
}

#ifdef __cplusplus
}
#endif
#endif
